@@ -1,7 +1,7 @@
 /*
  * This file is part of uudeview, the simple and friendly multi-part multi-
- * file uudecoder  program  (c)  1994 by Frank Pilhofer. The author may be
- * contacted by his email address,          fp@informatik.uni-frankfurt.de
+ * file uudecoder  program  (c) 1994-2001 by Frank Pilhofer. The author may
+ * be contacted at fp@fpx.de
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,9 +14,8 @@
  * GNU General Public License for more details.
  */
 
-
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef SYSTEM_WINDLL
@@ -54,7 +53,7 @@
 #include <fptools.h>
 #include <uustring.h>
 
-char * uucheck_id = "$Id: uucheck.c,v 1.1.1.1 1999/09/30 17:46:15 emessiah Exp $";
+char * uucheck_id = "$Id: uucheck.c,v 1.1.1.1 2006/10/02 20:45:58 csk Exp $";
 
 /*
  * Arbitrary number. This is the maximum number of part numbers we
@@ -776,7 +775,7 @@ UUPreProcessPart (fileread *data, int *ret)
     /*
      * in this case, it really _should_ have a filename somewhere
      */
-    if (result->filename != NULL)
+    if (result->filename != NULL && *result->filename)
       result->subfname = _FP_strdup (result->filename);
     else { /* if not, escape to UNKNOWN. We need to fill subfname */
       sprintf (temp, "%s.%03d", nofname, ++nofnum);
@@ -830,7 +829,7 @@ UUPreProcessPart (fileread *data, int *ret)
       /*
        * Assume it's the first part. I wonder why it's got no part number?
        */
-      if (result->filename != NULL)
+      if (result->filename != NULL && *result->filename)
         result->subfname = _FP_strdup (result->filename);
       else { /* if not, escape to UNKNOWN. We need to fill subfname */
         sprintf (temp, "%s.%03d", nofname, ++nofnum);
@@ -847,7 +846,7 @@ UUPreProcessPart (fileread *data, int *ret)
       }
       lastvalid = 0;
     }
-    else if (lastvalid && data->uudet == lastenc) {
+    else if (lastvalid && data->uudet == lastenc && result->partno == -1) {
       result->subfname = _FP_strdup (uucheck_lastname);
       result->partno   = ++lastpart;
 
@@ -857,6 +856,9 @@ UUPreProcessPart (fileread *data, int *ret)
       if (data->end || (data->partno && data->partno == data->maxpno))
 	lastvalid = 0;
     }
+    else if (data->partno != -1 && result->filename) {
+      result->subfname = _FP_strdup (result->filename);
+    }
     else { 
       /* 
        * it's got no info, it's got no begin, and we don't know anything
@@ -974,13 +976,13 @@ UUInsertPartToList (uufile *data)
 
   /*
    * Part belongs together, if
-   * (a) The file name received from the subject lines match _or_
-   *     the MIME-IDs match,
-   * (b) Not both parts have a begin line
-   * (c) Not both parts have an end line
-   * (d) Both parts don't have different MIME-IDs
-   * (e) Both parts don't encode different files
-   * (f) The other part wants to stay alone (FL_SINGLE)
+   * (1) The MIME-IDs match, or
+   * (2) The file name received from the subject lines match, and
+   *     (a) Not both parts have a begin line
+   *     (b) Not both parts have an end line
+   *     (c) Both parts don't have different MIME-IDs
+   *     (d) Both parts don't encode different files
+   *     (e) The other part wants to stay alone (FL_SINGLE)
    */
 
   /*
@@ -992,29 +994,33 @@ UUInsertPartToList (uufile *data)
     if (data->data->flags & FL_SINGLE) {
       /* this space intentionally left blank */
     }
-    else if ((_FP_stricmp (data->subfname, iter->subfname) == 0 ||
-         (data->mimeid && iter->mimeid &&
-          strcmp (data->mimeid, iter->mimeid) == 0)) &&
-	!(iter->begin && data->data->begin) &&
-	!(iter->end   && data->data->end) &&
-	!(data->mimeid && iter->mimeid &&
-	  strcmp (data->mimeid, iter->mimeid) != 0) &&
-	!(data->filename && iter->filename &&
-	  strcmp (data->filename, iter->filename) != 0) &&
-	!(iter->flags & FL_SINGLE)) {
+    else if ((data->mimeid && iter->mimeid &&
+	      strcmp (data->mimeid, iter->mimeid) == 0) ||
+	     (_FP_stricmp (data->subfname, iter->subfname) == 0 &&
+	      !(iter->begin && data->data->begin) &&
+	      !(iter->end   && data->data->end) &&
+	      !(data->mimeid && iter->mimeid &&
+		strcmp (data->mimeid, iter->mimeid) != 0) &&
+	      !(data->filename && iter->filename &&
+		strcmp (data->filename, iter->filename) != 0) &&
+	      !(iter->flags & FL_SINGLE))) {
 
       /*
-       * if we already have this part, don't try to insert it
+       * Don't insert a part that is already there.
+       *
+       * Also don't add a part beyond the "end" marker (unless we
+       * have a mimeid, which screws up the marker).
        */
 
-      for (fiter=iter->thisfile;
-	   fiter && (data->partno>fiter->partno) && !fiter->data->end;
-	   fiter=fiter->NEXT)
-	/* empty loop */ ;
-      if (fiter && 
-	  (data->partno==fiter->partno || 
-	   (data->partno > fiter->partno && fiter->data->end)))
-	goto goahead;
+      for (fiter=iter->thisfile; fiter; fiter=fiter->NEXT) {
+	if (data->partno == fiter->partno)
+	  goto goahead;
+	if (!iter->mimeid) {
+	  if (data->partno > fiter->partno && fiter->data->end) {
+	    goto goahead;
+	  }
+	}
+      }
 
       if (iter->filename == NULL && data->filename != NULL) {
         if ((iter->filename = _FP_strdup (data->filename)) == NULL)
@@ -1294,8 +1300,9 @@ UUCheckGlobalList (void)
 
     if ((part = fiter->partno) > 1) {
       if (!fiter->data->begin) {
-	for (count=1; count < part && miscount < MAXPLIST; count++)
+	for (count=1; count < part && miscount < MAXPLIST; count++) {
 	  misparts[miscount++] = count;
+        }
       }
     }
 
@@ -1460,9 +1467,3 @@ UUCheckGlobalList (void)
   return UUGlobalFileList;
 }
 
-
-/*****************************************************************************
- + Frank Pilhofer                             fp@informatik.uni-frankfurt.de +
- +---------------------------------------------------------------------------+
- | Department of Computer Sciences * University of Frankfurt / Main, Germany |
- *****************************************************************************/
