@@ -1,7 +1,7 @@
 /*
  * This file is part of uudeview, the simple and friendly multi-part multi-
- * file uudecoder  program  (c)  1994 by Frank Pilhofer. The author may be
- * contacted by his email address,          fp@informatik.uni-frankfurt.de
+ * file uudecoder  program  (c) 1994-2001 by Frank Pilhofer. The author may
+ * be contacted at fp@fpx.de
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef SYSTEM_WINDLL
@@ -81,7 +81,7 @@
 #include <fptools.h>
 #include <uustring.h>
 
-char * uulib_id = "$Id: uulib.c,v 1.1.1.1 1999/09/30 17:46:15 emessiah Exp $";
+char * uulib_id = "$Id: uulib.c,v 1.1.1.1 2006/10/02 20:45:58 csk Exp $";
 
 #ifdef SYSTEM_WINDLL
 BOOL _export WINAPI 
@@ -141,6 +141,8 @@ int uu_ignmode = 0;		/* ignore the original file mode            */
 int uu_handletext = 0;		/* do we want text/plain messages           */
 int uu_usepreamble = 0;		/* do we want Mime preambles/epilogues      */
 int uu_tinyb64 = 0;		/* detect short B64 outside of MIME         */
+int uu_remove_input = 0;        /* remove input files after decoding        */
+int uu_more_mime = 0;           /* strictly adhere to MIME headers          */
 
 headercount hlcount = {
   3,			        /* restarting after a MIME body             */
@@ -152,7 +154,7 @@ headercount hlcount = {
  * version string
  */
 
-char uulibversion[256] = "0.5pl13";
+char uulibversion[256] = "0.5pl20";
 
 /*
  * prefix to the files on disk, usually a path name to save files to
@@ -224,29 +226,30 @@ typedef struct {
 } allomap;
 
 static allomap toallocate[] = {
-  { &uugen_fnbuffer,    1024 },  /* generic filename buffer */
+  { &uugen_fnbuffer,    UUGEN_FNBUFFER_LEN },  /* generic filename buffer */
   { &uugen_inbuffer,    1024 },  /* generic input data buffer */
   { &uucheck_lastname,   256 },	 /* from uucheck.c */
   { &uucheck_tempname,   256 },
   { &uuestr_itemp,       256 },  /* from uuencode.c:UUEncodeStream() */
-  { &uuestr_otemp,       256 },
+  { &uuestr_otemp,      1024 },
   { &uulib_msgstring,   1024 },  /* from uulib.c:UUMessage() */
-  { &uuncdl_fulline,     256 },  /* from uunconc.c:UUDecodeLine() */
-  { &uuncdp_oline,       512 },  /* from uunconc.c:UUDecodePart() */
+  { &uuncdl_fulline,     300 },  /* from uunconc.c:UUDecodeLine() */
+  { &uuncdp_oline,      1200 },  /* from uunconc.c:UUDecodePart() */
   { &uunconc_UUxlat,     256 * sizeof (int) },  /* from uunconc.c:toplevel */
   { &uunconc_UUxlen,      64 * sizeof (int) },
   { &uunconc_B64xlat,    256 * sizeof (int) },
   { &uunconc_XXxlat,     256 * sizeof (int) },
   { &uunconc_BHxlat,     256 * sizeof (int) },
-  { &uunconc_save,     3*256 },  /* from uunconc.c:decoding buffer */
+  { &uunconc_save,     3*300 },  /* from uunconc.c:decoding buffer */
   { &uuscan_shlline,    1024 },  /* from uuscan.c:ScanHeaderLine() */
-  { &uuscan_pvvalue,     256 },  /* from uuscan.c:ParseValue() */
-  { &uuscan_phtext,      256 },  /* from uuscan.c:ParseHeader() */
-  { &uuscan_sdline,      256 },  /* from uuscan.c:ScanData() */
-  { &uuscan_sdbhds1,     256 },
-  { &uuscan_sdbhds2,     256 },
-  { &uuscan_spline,      256 },  /* from uuscan.c:ScanPart() */
-  { &uuutil_bhwtmp,      256 },  /* from uuutil.c:UUbhwrite() */
+  { &uuscan_shlline2,   1024 },  /* from uuscan.c:ScanHeaderLine() */
+  { &uuscan_pvvalue,    4096 },  /* from uuscan.c:ParseValue() */
+  { &uuscan_phtext,     4096 },  /* from uuscan.c:ParseHeader() */
+  { &uuscan_sdline,    UUSCAN_SDLINE_LEN },  /* from uuscan.c:ScanData() */
+  { &uuscan_sdbhds1,     300 },
+  { &uuscan_sdbhds2,     300 },
+  { &uuscan_spline,    UUSCAN_SPLINE_LEN },  /* from uuscan.c:ScanPart() */
+  { &uuutil_bhwtmp,      300 },  /* from uuutil.c:UUbhwrite() */
   { NULL, 0 }
 };
 
@@ -254,15 +257,15 @@ static allomap toallocate[] = {
  * Handle the printing of messages. Works like printf.
  */
 
-//#if (defined(STDC_HEADERS) || defined(HAVE_STDARG_H))
+#if defined(STDC_HEADERS) || defined(HAVE_STDARG_H)
 int
-UUMessage (char *file, int line, int level, char *format, ...) {
-//#else
-//int UUMessage (va_alist)
-//{
-//  va_dcl
-//#endif 
-//{
+UUMessage (char *file, int line, int level, char *format, ...)
+#else
+int
+UUMessage (va_alist)
+  va_dcl
+#endif
+{
   char *msgptr;
 #if defined(STDC_HEADERS) || defined(HAVE_STDARG_H)
   va_list ap;
@@ -487,6 +490,14 @@ UUGetOption (int option, int *ivalue, char *cvalue, int clength)
     _FP_strncpy (cvalue, uuencodeext, clength);
     result = 0;
     break;
+  case UUOPT_REMOVE:
+    if (ivalue) *ivalue = uu_remove_input;
+    result = uu_remove_input;
+    break;
+  case UUOPT_MOREMIME:
+    if (ivalue) *ivalue = uu_more_mime;
+    result = uu_more_mime;
+    break;
   default:
     return -1;
   }
@@ -541,6 +552,12 @@ UUSetOption (int option, int ivalue, char *cvalue)
     _FP_free (uuencodeext);
     uuencodeext = _FP_strdup (cvalue);
     break;
+  case UUOPT_REMOVE:
+    uu_remove_input = ivalue;
+    break;
+  case UUOPT_MOREMIME:
+    uu_more_mime = ivalue;
+    break;
   default:
     return UURET_ILLVAL;
   }
@@ -638,6 +655,12 @@ UUFNameFilter (char *fname)
 
 int UUEXPORT
 UULoadFile (char *filename, char *fileid, int delflag)
+{
+  return UULoadFileWithPartNo(filename, fileid, delflag, -1);
+}
+
+int UUEXPORT
+UULoadFileWithPartNo (char *filename, char *fileid, int delflag, int partno)
 {
   int res, sr, count=0;
   struct stat finfo;
@@ -732,7 +755,11 @@ UULoadFile (char *filename, char *fileid, int delflag)
       return UURET_IOERR;
     }
 
+    if (partno != -1)
+      loaded->partno = partno;
+
     if ((loaded->uudet == QP_ENCODED || loaded->uudet == PT_ENCODED) &&
+	(loaded->filename == NULL || *(loaded->filename) == '\0') &&
 	!uu_handletext && (loaded->flags&FL_PARTIAL)==0) {
       /*
        * Don't want text
@@ -812,6 +839,15 @@ UULoadFile (char *filename, char *fileid, int delflag)
     if (loaded->uudet)
       count++;
   }
+  if (ferror (datei)) {
+    UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
+	       uustring (S_READ_ERROR), filename,
+	       strerror (uu_errno = errno));
+    UUCheckGlobalList ();
+    progress.action = 0;
+    fclose (datei);
+    return UURET_IOERR;
+  }
   fclose (datei);
 
   if (!uu_fast_scanning && count==0) {
@@ -987,8 +1023,14 @@ UUDecodeFile (uulist *thefile, char *destname)
     }
   }
 
-  fclose (target);
   fclose (source);
+  if (fclose (target)) {
+    UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
+	       uustring (S_WR_ERR_TARGET),
+	       uugen_fnbuffer, strerror (uu_errno = errno));
+    unlink (uugen_fnbuffer);
+    return UURET_IOERR;
+  }
 
   /*
    * after a successful decoding run, we delete the temporary file
@@ -1077,6 +1119,9 @@ UUInfoFile (uulist *thefile, void *opaque,
     else if ((thefile->uudet == UU_ENCODED || thefile->uudet == XX_ENCODED) &&
 	     strncmp (uugen_inbuffer, "begin ", 6) == 0)
       break;
+    else if (thefile->uudet == YENC_ENCODED &&
+	     strncmp (uugen_inbuffer, "=ybegin ", 8) == 0)
+      break;
 
     if ((*func) (opaque, uugen_inbuffer))
       break;
@@ -1147,14 +1192,14 @@ int UUEXPORT
 UUCleanUp (void)
 {
   itbd *iter=ftodel, *ptr;
+  uulist *liter;
+  uufile *fiter;
   allomap *aiter;
 
-  UUkilllist (UUGlobalFileList);
-  UUGlobalFileList = NULL;
-
   /*
-   * delete input files
+   * delete temporary input files (such as the copy of stdin)
    */
+
   while (iter) {
     if (unlink (iter->fname)) {
       UUMessage (uulib_id, __LINE__, UUMSG_WARNING,
@@ -1166,8 +1211,36 @@ UUCleanUp (void)
     iter = iter->NEXT;
     _FP_free (ptr);
   }
+
   ftodel = NULL;
 
+  /*
+   * Delete input files after successful decoding
+   */
+
+  if (uu_remove_input) {
+    liter = UUGlobalFileList;
+    while (liter) {
+      if (liter->state & UUFILE_DECODED) {
+	fiter = liter->thisfile;
+	while (fiter) {
+	  if (fiter->data && fiter->data->sfname) {
+	    /*
+	     * Error code ignored. We might want to delete a file multiple
+	     * times
+	     */
+	    unlink (fiter->data->sfname);
+	  }
+	  fiter = fiter->NEXT;
+	}
+      }
+      liter = liter->NEXT;
+    }
+  }
+
+  UUkilllist (UUGlobalFileList);
+  UUGlobalFileList = NULL;
+
   _FP_free (uusavepath);
   _FP_free (uuencodeext);
   _FP_free (sstate.source);
