@@ -1,7 +1,7 @@
 /*
  * This file is part of uudeview, the simple and friendly multi-part multi-
- * file uudecoder  program  (c)  1994 by Frank Pilhofer. The author may be
- * contacted by his email address,          fp@informatik.uni-frankfurt.de
+ * file uudecoder  program  (c) 1994-2001 by Frank Pilhofer. The author may
+ * be contacted at fp@fpx.de
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,9 +25,11 @@
  **/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
+#include <glib.h>
+
 #ifdef SYSTEM_WINDLL
 #include <windows.h>
 #endif
@@ -49,12 +51,13 @@
 #include <errno.h>
 #endif
 
+#include <crc32.h>
 #include <uudeview.h>
 #include <uuint.h>
 #include <fptools.h>
 #include <uustring.h>
 
-char * uunconc_id = "$Id: uunconc.c,v 1.3 2000/01/16 07:44:20 emessiah Exp $";
+char * uunconc_id = "$Id: uunconc.c,v 1.1.1.1 2006/10/02 20:45:56 csk Exp $";
 
 /* for braindead systems */
 #ifndef SEEK_SET
@@ -165,7 +168,7 @@ UUInitConc (void)
 
   /* prepare line length table */
   UUxlen[0] = 1;
-  for(i = 1, j = 5; i <= 60; i += 3, j += 4)
+  for(i = 1, j = 5; i <= 61; i += 3, j += 4)
     UUxlen[i] = UUxlen[i+1] = UUxlen[i+2] = j;
 
   /* prepare other tables */
@@ -186,7 +189,7 @@ UUInitConc (void)
  * href statement
  */
 
-int
+static int
 UUBrokenByNetscape (char *string)
 {
   char *ptr;
@@ -229,7 +232,7 @@ UUBrokenByNetscape (char *string)
  * the second one for hrefs.
  */
 
-int
+static int
 UUNetscapeCollapse (char *string)
 {
   char *p1=string, *p2=string;
@@ -243,11 +246,10 @@ UUNetscapeCollapse (char *string)
    */
   while (*p1) {
     if (*p1 == '&') {
-      if      (_FP_strnicmp (p1, "&amp;", 5) == 0) { p1+=5; *p2++='&'; }
-      else if (_FP_strnicmp (p1, "&lt;",  4) == 0) { p1+=4; *p2++='<'; }
-      else if (_FP_strnicmp (p1, "&gt;",  4) == 0) { p1+=4; *p2++='>'; }
+      if      (_FP_strnicmp (p1, "&amp;", 5) == 0) { p1+=5; *p2++='&'; res=1; }
+      else if (_FP_strnicmp (p1, "&lt;",  4) == 0) { p1+=4; *p2++='<'; res=1; }
+      else if (_FP_strnicmp (p1, "&gt;",  4) == 0) { p1+=4; *p2++='>'; res=1; }
       else *p2++ = *p1++;
-      res = 1;
     }
     else *p2++ = *p1++;
   }
@@ -298,10 +300,10 @@ UUValidData (char *ptr, int encoding, int *bhflag)
   int i=0, j, len=0, suspicious=0, flag=0;
   char *s = ptr;
 
-  if ((s == NULL) || (*s < '\0')) {
-    return(0);              /* bad string */
+  if ((s == NULL) || (*s == '\0')) {
+    return 0;              /* bad string */
   }
-  
+
   while (*s && *s!='\012' && *s!='\015') {
     s++;
     len++;
@@ -320,6 +322,8 @@ UUValidData (char *ptr, int encoding, int *bhflag)
     goto _t_B64;
   case BH_ENCODED:
     goto _t_Binhex;
+  case YENC_ENCODED:
+    return YENC_ENCODED;
   }
 
  _t_Binhex:                 /* Binhex Test */
@@ -450,8 +454,10 @@ UUValidData (char *ptr, int encoding, int *bhflag)
    * evaluated if the first character is lowercase, which really shouldn't
    * be in uuencoded text.
    */
-  if (len != j && 
-      !(*ptr != 'M' && *ptr != 'h' && len > j && len <= UUxlen[UUxlat['M']])) {
+  if (len != j &&
+      ((ptr[0] == '-' && ptr[1] == '-' && strstr(ptr,"part")!=NULL) ||
+       !(*ptr != 'M' && *ptr != 'h' &&
+	 len > j && len <= UUxlen[UUxlat['M']]))) {
     if (encoding==UU_ENCODED) return 0;
     goto _t_XX;             /* bad length */
   }
@@ -537,6 +543,8 @@ UURepairData (FILE *datei, char *line, int encoding, int *bhflag)
 
   while (vflag == 0 && nflag && safety--) {
     if (nflag == 1) {		/* need next line to repair */
+      if (strlen (line) > 250)
+	break;
       ptr = line + strlen (line);
       while (ptr>line && (*(ptr-1)=='\015' || *(ptr-1)=='\012'))
 	ptr--;
@@ -616,26 +624,26 @@ UUDecodeLine (char *s, char *d, int method)
     else
       table = XXxlat;
 
-    i = table [*s++];
+    i = table [ACAST(*s++)];
     j = UUxlen[i] - 1;
 
     while(j > 0) {
-      c  = table[*s++] << 2;
-      cc = table[*s++];
+      c  = table[ACAST(*s++)] << 2;
+      cc = table[ACAST(*s++)];
       c |= (cc >> 4);
 
       if(i-- > 0)
 	d[count++] = c;
       
       cc <<= 4;
-      c    = table[*s++];
+      c    = table[ACAST(*s++)];
       cc  |= (c >> 2);
       
       if(i-- > 0)
 	d[count++] = cc;
       
       c <<= 6;
-      c |= table[*s++];
+      c |= table[ACAST(*s++)];
       
       if(i-- > 0)
 	d[count++] = c;
@@ -705,6 +713,22 @@ UUDecodeLine (char *s, char *d, int method)
     while (BHxlat[ACAST(*s)] != -1)
       uuncdl_fulline[leftover++] = *s++;
   }
+  else if (method == YENC_ENCODED) {
+    while (*s) {
+      if (*s == '=') {
+	if (*++s != '\0') {
+	  d[count++] = (char) ((int) *s - 64 - 42);
+	  s++;
+	}
+      }
+      else if (*s == '\n' || *s == '\r') {
+	s++; /* ignore */
+      }
+      else {
+	d[count++] = (char) ((int) *s++ - 42);
+      }
+    }
+  }
 
   return count;
 }
@@ -713,7 +737,7 @@ UUDecodeLine (char *s, char *d, int method)
  * ``Decode'' Quoted-Printable text
  */
 
-int
+static int
 UUDecodeQP (FILE *datain, FILE *dataout, int *state,
 	    long maxpos, int method, int flags,
 	    char *boundary)
@@ -811,7 +835,7 @@ UUDecodeQP (FILE *datain, FILE *dataout, int *state,
  * ``Decode'' plain text. Our job is to properly handle the EOL sequence
  */
 
-int
+static int
 UUDecodePT (FILE *datain, FILE *dataout, int *state,
 	    long maxpos, int method, int flags,
 	    char *boundary)
@@ -861,9 +885,8 @@ UUDecodePT (FILE *datain, FILE *dataout, int *state,
      * So if the part ends here, don't print a line break"
      */
     if ((*ptr == '\012' || *ptr == '\015') &&
-	(!feof (datain) && 
-	 (ftell(datain)<maxpos || flags&FL_TOEND || flags&FL_PARTIAL ||
-	  (!(flags&FL_PROPER) && uu_fast_scanning)))) {
+	(ftell(datain)<maxpos || flags&FL_TOEND || flags&FL_PARTIAL ||
+	 !boundary || (!(flags&FL_PROPER) && uu_fast_scanning))) {
       *ptr = '\0';
       fprintf (dataout, "%s\n", line);
     }
@@ -875,18 +898,89 @@ UUDecodePT (FILE *datain, FILE *dataout, int *state,
   return UURET_OK;
 }
 
+/*
+ * Decode a single field using method. For the moment, this supports
+ * Base64 and Quoted Printable only, to support RFC 1522 header decoding.
+ * Quit when seeing the RFC 1522 ?= end marker.
+ */
+
+int
+UUDecodeField (char *s, char *d, int method)
+{
+  int z1, z2, z3, z4;
+  int count=0;
+
+  if (method == B64ENCODED) {
+    while ((z1 = B64xlat[ACAST(*s)]) != -1) {
+      if ((z2 = B64xlat[ACAST(*(s+1))]) == -1) break;
+      if ((z3 = B64xlat[ACAST(*(s+2))]) == -1) break;
+      if ((z4 = B64xlat[ACAST(*(s+3))]) == -1) break;
+
+      d[count++] = (z1 << 2) | (z2 >> 4);
+      d[count++] = (z2 << 4) | (z3 >> 2);
+      d[count++] = (z3 << 6) | (z4);
+
+      s+=4;
+    }
+    if (z1 != -1 && z2 != -1 && *(s+2) == '=') {
+      d[count++] = (z1 << 2) | (z2 >> 4);
+      s+=2;
+    }
+    else if (z1 != -1 && z2 != -1 && z3 != -1 && *(s+3) == '=') {
+      d[count++] = (z1 << 2) | (z2 >> 4);
+      d[count++] = (z2 << 4) | (z3 >> 2);
+      s+=3;
+    }
+  }
+  else if (method == QP_ENCODED) {
+    while (*s && (*s != '?' || *(s+1) != '=')) {
+      while (*s && *s != '=' && (*s != '?' || *(s+1) != '=')) {
+	d[count++] = *s++;
+      }
+      if (*s == '=') {
+	if (isxdigit (*(s+1)) && isxdigit (*(s+2))) {
+	  d[count]  = (isdigit (*(s+1)) ? (*(s+1)-'0') : (tolower (*(s+1))-'a'+10)) << 4;
+	  d[count] |= (isdigit (*(s+2)) ? (*(s+2)-'0') : (tolower (*(s+2))-'a'+10));
+	  count++;
+	  s+=3;
+	}
+	else if (*(s+1) == '\012' || *(s+1) == '\015') {
+	  s+=2;
+	}
+	else {
+	  d[count++] = *s++;
+	}
+      }
+    }
+  }
+  else {
+    return -1;
+  }
+
+  d[count] = '\0';
+  return count;
+}
+
 int
 UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	      long maxpos, int method, int flags,
 	      char *boundary)
 {
-  char *line=uugen_fnbuffer, *oline=uuncdp_oline;
+  char *line = uugen_fnbuffer;
+  int line_len = UUGEN_FNBUFFER_LEN;
+  char *oline=uuncdp_oline;
   int warning=0, vlc=0, lc[2], hadct=0;
   int tc=0, tf=0, vflag, haddata=0, haddh=0;
+  long yefilesize=0, yepartends=0;
+  crc32_t yepartcrc=crc32(0L, Z_NULL, 0);
+  static crc32_t yefilecrc=0;
   static int bhflag=0;
   size_t count=0;
+  size_t yepartsize=0;
+  char *ptr;
 
   if (datain == NULL || dataout == NULL) {
+    yefilecrc = crc32(0L, Z_NULL, 0);
     bhflag = 0;
     return UURET_OK;
   }
@@ -907,27 +1001,35 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 
   uulboundary = -1;
 
+  if (method == YENC_ENCODED) {
+    *state = BEGIN;
+  }
+
   while (!feof (datain) && *state != DONE && 
 	 (ftell(datain)<maxpos || flags&FL_TOEND || maxpos==-1 ||
 	  (!(flags&FL_PROPER) && uu_fast_scanning))) {
-    if (_FP_fgets (line, 255, datain) == NULL)
+    if (_FP_fgets (line, line_len, datain) == NULL)
       break;
+
     if (ferror (datain)) {
       UUMessage (uunconc_id, __LINE__, UUMSG_ERROR,
 		 uustring (S_SOURCE_READ_ERR),
 		 strerror (uu_errno = errno));
       return UURET_IOERR;
     }
+
     if (line[0]=='\015' || line[0]=='\012') { /* Empty line? */
       if (*state == DATA &&
 	  (method == UU_ENCODED || method == XX_ENCODED))
 	*state = END;
+
       /*
        * if we had a whole block of valid lines before, we reset our
        * 'valid data' flag, tf. Without this 'if', we'd break decoding
        * files with interleaved blank lines. The value of 5 is chosen
        * quite arbitrarly.
        */
+
       if (vlc > 5)
 	tf = tc = 0;
       vlc = 0;
@@ -987,8 +1089,9 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
     }
 
     if (*state == BEGIN) {
-      if (strncmp      (line, "begin ",       6) == 0 ||
-	  _FP_strnicmp (line, "<pre>begin ", 11) == 0) {    /* for LYNX */
+      if ((method == UU_ENCODED || method == XX_ENCODED) &&
+	  (strncmp      (line, "begin ",       6) == 0 ||
+	   _FP_strnicmp (line, "<pre>begin ", 11) == 0)) { /* for LYNX */
 	*state = DATA;
 	continue;
       }
@@ -1000,8 +1103,36 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	else
 	  continue;
       }
-      else
+      else if (method == YENC_ENCODED &&
+	       strncmp (line, "=ybegin ", 8) == 0 &&
+	       _FP_strstr (line, " name=") != NULL) {
+	*state = DATA;
+
+	if ((ptr = _FP_strstr (line, " size=")) != NULL) {
+	  ptr += 6;
+	  yefilesize = atoi (ptr);
+	}
+	else {
+	  yefilesize = -1;
+	}
+
+	if (_FP_strstr (line, " part=") != NULL) {
+	  if (_FP_fgets (line, line_len, datain) == NULL) {
+	    break;
+	  }
+
+	  if ((ptr = _FP_strstr (line, " end=")) == NULL) {
+	    break;
+	  }
+       
+	  yepartends = atoi (ptr + 5);
+	}
+	tf = 1;
 	continue;
+      }
+      else {
+	continue;
+      }
       
       tc = tf = vlc = 0;
       lc[0] = lc[1] = 0;
@@ -1013,6 +1144,44 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	break;
       }
     }
+
+    if (*state == DATA && method == YENC_ENCODED &&
+	strncmp (line, "=yend ", 6) == 0) {
+      if ((ptr = _FP_strstr (line, " pcrc32=")) != NULL) {
+	crc32_t pcrc32 = strtoul (ptr + 8, NULL, 16);
+	if (pcrc32 != yepartcrc) {
+	  UUMessage (uunconc_id, __LINE__, UUMSG_WARNING,
+		     uustring (S_PCRC_MISMATCH), progress.curfile, progress.partno);
+	}
+      }
+      if ((ptr = _FP_strstr (line, " crc32=")) != NULL)
+      {
+	crc32_t fcrc32 = strtoul (ptr + 7, NULL, 16);
+	if (fcrc32 != yefilecrc) {
+	  UUMessage (uunconc_id, __LINE__, UUMSG_WARNING,
+		     uustring (S_CRC_MISMATCH), progress.curfile);
+	}
+      }
+      if ((ptr = _FP_strstr (line, " size=")) != NULL)
+      {
+	size_t size = atol(ptr + 6);
+	if (size != yepartsize && yefilesize != -1) {
+	  if (size != yefilesize)
+	    UUMessage (uunconc_id, __LINE__, UUMSG_WARNING,
+		       uustring (S_PSIZE_MISMATCH), progress.curfile,
+		       progress.partno, yepartsize, size);
+	  else
+	    UUMessage (uunconc_id, __LINE__, UUMSG_WARNING,
+		       uustring (S_SIZE_MISMATCH), progress.curfile,
+		       yepartsize, size);
+	}
+      }
+      if (yepartends == 0 || yepartends >= yefilesize) {
+	*state = DONE;
+      }
+      break;
+    }
+
     if (*state == DATA || *state == END) {
       if (method==B64ENCODED && line[0]=='-' && line[1]=='-' && tc) {
 	break;
@@ -1034,6 +1203,12 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
       if (vflag == method) {
 	if (tf) {
 	  count  = UUDecodeLine (line, oline, method);
+	  if (method == YENC_ENCODED) {
+	    if (yepartends)
+	      yepartcrc = crc32(yepartcrc, oline, count);
+	    yefilecrc = crc32(yefilecrc, oline, count);
+	    yepartsize += count;
+	  }
 	  vlc++; lc[1]++;
 	}
 	else if (tc == 3) {
@@ -1061,6 +1236,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	else {
 	  _FP_strncpy (save[tc++], line, 256);
 	}
+
 	if (method == UU_ENCODED)
 	  *state = (line[0] == 'M') ? DATA : END;
 	else if (method == XX_ENCODED)
@@ -1079,6 +1255,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
     else if (*state != DONE) {
       return UURET_NOEND;
     }
+
     if (count) {
       if (method == BH_ENCODED) {
 	if (UUbhwrite (oline, 1, count, dataout) != count) {
@@ -1135,10 +1312,10 @@ UUDecode (uulist *data)
   long rsize, dsize, numbytes;
   FILE *datain, *dataout;
   unsigned char r[8];
-  char *mode, *ntmp;
+  char *mode, *ntmp, *template;
   uufile *iter;
   size_t bytes;
-  char *pan_tmp_dir = NULL;
+  int fd;
 
   if (data == NULL || data->thisfile == NULL)
     return UURET_ILLVAL;
@@ -1152,20 +1329,27 @@ UUDecode (uulist *data)
   if (data->state & UUFILE_NOBEGIN && !uu_desperate)
     return UURET_NODATA;
 
-  if (data->uudet == QP_ENCODED || data->uudet == PT_ENCODED)
+  if (data->uudet == PT_ENCODED)
     mode = "wt";	/* open text files in text mode */
   else
     mode = "wb";	/* otherwise in binary          */
 
-  
-  if ((data->binfile = tempnam (NULL, "uu")) == NULL) {
+
+  template = g_build_filename (g_get_tmp_dir(), "uuXXXXXX", NULL);
+  data->binfile = strdup (template);
+  fd = g_mkstemp (data->binfile);
+  g_free (template);
+
+  if (fd == -1) {
     UUMessage (uunconc_id, __LINE__, UUMSG_ERROR,
 	       uustring (S_NO_TEMP_NAME));
+    _FP_free (data->binfile);
+    data->binfile = NULL;
+    uu_errno = errno;
     return UURET_NOMEM;
   }
-  g_free (pan_tmp_dir);
 
-  if ((dataout = fopen (data->binfile, mode)) == NULL) {
+  if ((dataout = fdopen (fd, mode)) == NULL) {
     /*
      * we couldn't create a temporary file. Usually this means that TMP
      * and TEMP aren't set
@@ -1231,7 +1415,7 @@ UUDecode (uulist *data)
 
   iter = data->thisfile;
   while (iter) {
-    if (part != -1 && iter->partno != part+1)
+    if (part != -1 && iter->partno != part+1 && !uu_desperate)
       break;
     else
       part = iter->partno;
@@ -1295,7 +1479,12 @@ UUDecode (uulist *data)
        data->uudet == PT_ENCODED))
     state = DONE; /* assume we're done */
 
-  fclose (dataout);
+  if (fclose (dataout)) {
+    UUMessage (uunconc_id, __LINE__, UUMSG_ERROR,
+	       uustring (S_WR_ERR_TEMP),
+	       strerror (uu_errno = errno));
+    res = UURET_IOERR;
+  }
 
   if (res != UURET_OK || (state != DONE && !uu_desperate)) {
     unlink (data->binfile);
@@ -1320,14 +1509,21 @@ UUDecode (uulist *data)
    * If this was a BinHex file, we must extract its data or resource fork
    */
 
-  if (data->uudet == BH_ENCODED && data->binfile) {
-    if ((ntmp = tempnam (NULL, "uu")) == NULL) {
+  if (data->uudet == BH_ENCODED && data->binfile)
+  {
+    char * template = g_build_filename (g_get_tmp_dir(), "uuXXXXXX", NULL);
+    ntmp = strdup (template);
+    fd = g_mkstemp (ntmp);
+    g_free (template);
+
+    if (fd == -1) {
       UUMessage (uunconc_id, __LINE__, UUMSG_ERROR,
 		 uustring (S_NO_TEMP_NAME));
       progress.action = 0;
+      free (ntmp);
       return UURET_NOMEM;
     }
-    if ((datain = fopen (data->binfile, "rb")) == NULL) {
+    if ((datain = fdopen (fd, "rb")) == NULL) {
       UUMessage (uunconc_id, __LINE__, UUMSG_ERROR,
 		 uustring (S_NOT_OPEN_FILE),
 		 data->binfile, strerror (uu_errno = errno));
@@ -1445,7 +1641,14 @@ UUDecode (uulist *data)
      */
 
     fclose (datain);
-    fclose (dataout);
+    if (fclose (dataout)) {
+      UUMessage (uunconc_id, __LINE__, UUMSG_ERROR,
+		 uustring (S_WR_ERR_TARGET),
+		 ntmp, strerror (uu_errno = errno));
+      unlink (ntmp);
+      free   (ntmp);
+      return UURET_IOERR;
+    }
 
     if (unlink (data->binfile)) {
       UUMessage (uunconc_id, __LINE__, UUMSG_WARNING,
